package sun.net.www.protocol.http;

import java.security.AccessController;
import sun.security.action.GetBooleanAction;
import java.io.ObjectOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import sun.net.www.HeaderParser;
import java.net.URL;
import java.util.HashMap;
import java.net.PasswordAuthentication;

public abstract class AuthenticationInfo extends AuthCacheValue implements Cloneable
{
    static final long serialVersionUID = -2588378268010453259L;
    public static final char SERVER_AUTHENTICATION = 's';
    public static final char PROXY_AUTHENTICATION = 'p';
    static final boolean serializeAuth;
    protected transient PasswordAuthentication pw;
    private static HashMap<String, Thread> requests;
    char type;
    AuthScheme authScheme;
    String protocol;
    String host;
    int port;
    String realm;
    String path;
    String s1;
    String s2;
    
    public PasswordAuthentication credentials() {
        return this.pw;
    }
    
    public AuthCacheValue.Type getAuthType() {
        return (this.type == 's') ? AuthCacheValue.Type.Server : AuthCacheValue.Type.Proxy;
    }
    
    @Override
    AuthScheme getAuthScheme() {
        return this.authScheme;
    }
    
    public String getHost() {
        return this.host;
    }
    
    public int getPort() {
        return this.port;
    }
    
    public String getRealm() {
        return this.realm;
    }
    
    public String getPath() {
        return this.path;
    }
    
    public String getProtocolScheme() {
        return this.protocol;
    }
    
    protected boolean useAuthCache() {
        return true;
    }
    
    private static boolean requestIsInProgress(String key) {
        if (!AuthenticationInfo.serializeAuth) {
            return false;
        }
        synchronized (AuthenticationInfo.requests) {
            Thread currentThread = Thread.currentThread();
            Thread thread;
            if ((thread = (Thread)AuthenticationInfo.requests.get(key)) == null) {
                AuthenticationInfo.requests.put(key, currentThread);
                return false;
            }
            if (thread == currentThread) {
                return false;
            }
            while (AuthenticationInfo.requests.containsKey(key)) {
                try {
                    AuthenticationInfo.requests.wait();
                }
                catch (InterruptedException ex) {}
            }
        }
        return true;
    }
    
    private static void requestCompleted(String s) {
        synchronized (AuthenticationInfo.requests) {
            Thread thread = (Thread)AuthenticationInfo.requests.get(s);
            if (thread != null && thread == Thread.currentThread()) {
                int n = AuthenticationInfo.requests.remove(s) != null;
                assert !(n == 0);
            }
            AuthenticationInfo.requests.notifyAll();
        }
    }
    
    public AuthenticationInfo(char type, AuthScheme authScheme, String s, int port, String realm) {
        this.type = type;
        this.authScheme = authScheme;
        this.protocol = "";
        this.host = s.toLowerCase();
        this.port = port;
        this.realm = realm;
        this.path = null;
    }
    
    public Object clone() {
        try {
            return super.clone();
        }
        catch (CloneNotSupportedException ex) {
            return null;
        }
    }
    
    public AuthenticationInfo(char type, AuthScheme authScheme, URL url, String realm) {
        this.type = type;
        this.authScheme = authScheme;
        this.protocol = url.getProtocol().toLowerCase();
        this.host = url.getHost().toLowerCase();
        this.port = url.getPort();
        if (this.port == -1) {
            this.port = url.getDefaultPort();
        }
        this.realm = realm;
        String path = url.getPath();
        if (path.length() == 0) {
            this.path = path;
        }
        else {
            this.path = reducePath(path);
        }
    }
    
    static String reducePath(String s) {
        int lastIndex = s.lastIndexOf(47);
        int lastIndex2 = s.lastIndexOf(46);
        if (lastIndex != -1) {
            if (lastIndex < lastIndex2) {
                return s.substring(0, lastIndex + 1);
            }
            return s;
        }
        else {
            return s;
        }
    }
    
    static AuthenticationInfo getServerAuth(URL url) {
        int i = url.getPort();
        if (i == -1) {
            i = url.getDefaultPort();
        }
        return getAuth("s:" + url.getProtocol().toLowerCase() + ":" + url.getHost().toLowerCase() + ":" + i, url);
    }
    
    static String getServerAuthKey(URL url, String str, AuthScheme obj) {
        int i = url.getPort();
        if (i == -1) {
            i = url.getDefaultPort();
        }
        return "s:" + obj + ":" + url.getProtocol().toLowerCase() + ":" + url.getHost().toLowerCase() + ":" + i + ":" + str;
    }
    
    static AuthenticationInfo getServerAuth(String s) {
        AuthenticationInfo authenticationInfo = getAuth(s, null);
        if (authenticationInfo == null && requestIsInProgress(s)) {
            authenticationInfo = getAuth(s, null);
            return authenticationInfo;
        }
        return authenticationInfo;
    }
    
    static AuthenticationInfo getAuth(String s, URL url) {
        if (url == null) {
            return (AuthenticationInfo)AuthenticationInfo.cache.get(s, null);
        }
        return (AuthenticationInfo)AuthenticationInfo.cache.get(s, url.getPath());
    }
    
    static AuthenticationInfo getProxyAuth(String s, int i) {
        return (AuthenticationInfo)AuthenticationInfo.cache.get("p::" + s.toLowerCase() + ":" + i, null);
    }
    
    static String getProxyAuthKey(String s, int i, String str, AuthScheme obj) {
        return "p:" + obj + "::" + s.toLowerCase() + ":" + i + ":" + str;
    }
    
    static AuthenticationInfo getProxyAuth(String s) {
        AuthenticationInfo authenticationInfo = (AuthenticationInfo)AuthenticationInfo.cache.get(s, null);
        if (authenticationInfo == null && requestIsInProgress(s)) {
            authenticationInfo = (AuthenticationInfo)AuthenticationInfo.cache.get(s, null);
            return authenticationInfo;
        }
        return authenticationInfo;
    }
    
    void addToCache() {
        String cacheKey = this.cacheKey(true);
        if (this.useAuthCache()) {
            AuthenticationInfo.cache.put(cacheKey, this);
            if (this.supportsPreemptiveAuthorization()) {
                AuthenticationInfo.cache.put(this.cacheKey(false), this);
            }
        }
        endAuthRequest(cacheKey);
    }
    
    static void endAuthRequest(String s) {
        if (!AuthenticationInfo.serializeAuth) {
            return;
        }
        synchronized (AuthenticationInfo.requests) {
            requestCompleted(s);
        }
        return;
    }
    
    void removeFromCache() {
        AuthenticationInfo.cache.remove(this.cacheKey(true), this);
        if (this.supportsPreemptiveAuthorization()) {
            AuthenticationInfo.cache.remove(this.cacheKey(false), this);
        }
    }
    
    public abstract boolean supportsPreemptiveAuthorization();
    
    public String getHeaderName() {
        if (this.type == 's') {
            return "Authorization";
        }
        return "Proxy-authorization";
    }
    
    public abstract String getHeaderValue(URL p0, String p1);
    
    public abstract boolean setHeaders(HttpURLConnection p0, HeaderParser p1, String p2);
    
    public abstract boolean isAuthorizationStale(String p0);
    
    String cacheKey(boolean b) {
        if (b) {
            return this.type + ":" + this.authScheme + ":" + this.protocol + ":" + this.host + ":" + this.port + ":" + this.realm;
        }
        return this.type + ":" + this.protocol + ":" + this.host + ":" + this.port;
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws IOException, ClassNotFoundException {
        objectInputStream.defaultReadObject();
        this.pw = new PasswordAuthentication(this.s1, this.s2.toCharArray());
        this.s1 = null;
        this.s2 = null;
    }
    
    private synchronized void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
        this.s1 = this.pw.getUserName();
        this.s2 = new String(this.pw.getPassword());
        objectOutputStream.defaultWriteObject();
    }
    
    static {
        serializeAuth = (boolean)AccessController.doPrivileged(new GetBooleanAction("http.auth.serializeRequests"));
        AuthenticationInfo.requests = new HashMap();
    }
}
